@@ -2,10 +2,12 @@
 from .context import (
     Context,
     Repository,
+    BaseRepository,
     get_dataset,
     prepare_dataset,
 )
 
 from pkg_resources import get_distribution, DistributionNotFound
-
+from .definitions import dataset, metadata
+from .data import Base
 from .version import version, version_tuple
